@@ -1,5 +1,6 @@
 /*
- * Minio Go Library for Amazon S3 Compatible Cloud Storage (C) 2015 Minio, Inc.
+ * Minio Go Library for Amazon S3 Compatible Cloud Storage
+ * Copyright 2015-2017 Minio, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -36,16 +37,16 @@ type owner struct {
 	ID          string
 }
 
-// commonPrefix container for prefix response.
-type commonPrefix struct {
+// CommonPrefix container for prefix response.
+type CommonPrefix struct {
 	Prefix string
 }
 
-// listBucketResult container for listObjects V2 response.
-type listBucketV2Result struct {
+// ListBucketV2Result container for listObjects response version 2.
+type ListBucketV2Result struct {
 	// A response can contain CommonPrefixes only if you have
 	// specified a delimiter.
-	CommonPrefixes []commonPrefix
+	CommonPrefixes []CommonPrefix
 	// Metadata about each object returned.
 	Contents  []ObjectInfo
 	Delimiter string
@@ -70,11 +71,11 @@ type listBucketV2Result struct {
 	StartAfter string
 }
 
-// listBucketResult container for listObjects response.
-type listBucketResult struct {
+// ListBucketResult container for listObjects response.
+type ListBucketResult struct {
 	// A response can contain CommonPrefixes only if you have
 	// specified a delimiter.
-	CommonPrefixes []commonPrefix
+	CommonPrefixes []CommonPrefix
 	// Metadata about each object returned.
 	Contents  []ObjectInfo
 	Delimiter string
@@ -102,8 +103,8 @@ type listBucketResult struct {
 	Prefix     string
 }
 
-// listMultipartUploadsResult container for ListMultipartUploads response
-type listMultipartUploadsResult struct {
+// ListMultipartUploadsResult container for ListMultipartUploads response
+type ListMultipartUploadsResult struct {
 	Bucket             string
 	KeyMarker          string
 	UploadIDMarker     string `xml:"UploadIdMarker"`
@@ -116,7 +117,7 @@ type listMultipartUploadsResult struct {
 	Prefix             string
 	Delimiter          string
 	// A response can contain CommonPrefixes only if you specify a delimiter.
-	CommonPrefixes []commonPrefix
+	CommonPrefixes []CommonPrefix
 }
 
 // initiator container for who initiated multipart upload.
@@ -128,11 +129,11 @@ type initiator struct {
 // copyObjectResult container for copy object response.
 type copyObjectResult struct {
 	ETag         string
-	LastModified string // time string format "2006-01-02T15:04:05.000Z"
+	LastModified time.Time // time string format "2006-01-02T15:04:05.000Z"
 }
 
-// objectPart container for particular part of an object.
-type objectPart struct {
+// ObjectPart container for particular part of an object.
+type ObjectPart struct {
 	// Part number identifies the part.
 	PartNumber int
 
@@ -147,8 +148,8 @@ type objectPart struct {
 	Size int64
 }
 
-// listObjectPartsResult container for ListObjectParts response.
-type listObjectPartsResult struct {
+// ListObjectPartsResult container for ListObjectParts response.
+type ListObjectPartsResult struct {
 	Bucket   string
 	Key      string
 	UploadID string `xml:"UploadId"`
@@ -163,7 +164,7 @@ type listObjectPartsResult struct {
 
 	// Indicates whether the returned list of parts is truncated.
 	IsTruncated bool
-	ObjectParts []objectPart `xml:"Part"`
+	ObjectParts []ObjectPart `xml:"Part"`
 
 	EncodingType string
 }
@@ -185,9 +186,9 @@ type completeMultipartUploadResult struct {
 	ETag     string
 }
 
-// completePart sub container lists individual part numbers and their
+// CompletePart sub container lists individual part numbers and their
 // md5sum, part of completeMultipartUpload.
-type completePart struct {
+type CompletePart struct {
 	XMLName xml.Name `xml:"http://s3.amazonaws.com/doc/2006-03-01/ Part" json:"-"`
 
 	// Part number identifies the part.
@@ -198,7 +199,7 @@ type completePart struct {
 // completeMultipartUpload container for completing multipart upload.
 type completeMultipartUpload struct {
 	XMLName xml.Name       `xml:"http://s3.amazonaws.com/doc/2006-03-01/ CompleteMultipartUpload" json:"-"`
-	Parts   []completePart `xml:"Part"`
+	Parts   []CompletePart `xml:"Part"`
 }
 
 // createBucketConfiguration container for bucket configuration.
@@ -210,15 +211,16 @@ type createBucketConfiguration struct {
 // deleteObject container for Delete element in MultiObjects Delete XML request
 type deleteObject struct {
 	Key       string
-	VersionId string `xml:"VersionId,omitempty"`
+	VersionID string `xml:"VersionId,omitempty"`
 }
 
 // deletedObject container for Deleted element in MultiObjects Delete XML response
 type deletedObject struct {
-	Key                   string
-	VersionId             string `xml:"VersionId,omitempty"`
+	Key       string
+	VersionID string `xml:"VersionId,omitempty"`
+	// These fields are ignored.
 	DeleteMarker          bool
-	DeleteMarkerVersionId string
+	DeleteMarkerVersionID string
 }
 
 // nonDeletedObject container for Error element (failed deletion) in MultiObjects Delete XML response
