@@ -1,7 +1,8 @@
 // +build ignore
 
 /*
- * Minio Go Library for Amazon S3 Compatible Cloud Storage (C) 2015 Minio, Inc.
+ * Minio Go Library for Amazon S3 Compatible Cloud Storage
+ * Copyright 2015-2017 Minio, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -38,10 +39,14 @@ func main() {
 		log.Fatalln(err)
 	}
 
-	err = s3Client.BucketExists("my-bucketname")
+	found, err := s3Client.BucketExists("my-bucketname")
 	if err != nil {
 		log.Fatalln(err)
 	}
 
-	log.Println("Success")
+	if found {
+		log.Println("Bucket found.")
+	} else {
+		log.Println("Bucket not found.")
+	}
 }
